/*=============================================================================
Turing Machine Debugger
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
  -- License restrictions apply (Dual FAL/GPL2)
  -- For details see doc/FAL.txt and doc/GPL2.txt.
=============================================================================*/
#include <stdio.h>
#include "ops.h"

static struct iop m_adder[][MAX_CHARBIT] = {
           /* Input byte '0' */        /* Input byte '1' */
    [0] = {['0'] = {'1', 'R', 1}, /**/ ['1'] = {'1', 'L', 0}},
    [1] = {['0'] = {'0', 'L', 2}, /**/ ['1'] = {'1', 'R', 1}},
    [2] = {['0'] = {'0', 'E', 0}, /**/ ['1'] = {'0', 'L', 3}},
    [3] = {['0'] = {'0', 'E', 0}, /**/ ['1'] = {'0', 'H', 0}},
};

static struct iop m_copy2[][MAX_CHARBIT] = {
    [0] = {['0'] = {'0', 'H', 1}, /**/ ['1'] = {'0', 'R', 1}},
    [1] = {['0'] = {'0', 'R', 2}, /**/ ['1'] = {'1', 'R', 1}},
    [2] = {['0'] = {'1', 'L', 3}, /**/ ['1'] = {'1', 'R', 2}},
    [3] = {['0'] = {'0', 'L', 4}, /**/ ['1'] = {'1', 'L', 3}},
    [4] = {['0'] = {'1', 'L', 5}, /**/ ['1'] = {'1', 'L', 4}},
    [5] = {['0'] = {'0', 'R', 6}, /**/ ['1'] = {'0', 'R', 1}},
    [6] = {['0'] = {'0', 'R', 7}, /**/ ['1'] = {'1', 'R', 6}},
    [7] = {['0'] = {'0', 'H', 0}, /**/ ['1'] = {'1', 'R', 7}},
};

static struct iop m_copy[][MAX_CHARBIT] = {
    /* State,         Input byte == '0',             Input byte == '1' */
    [   0] = {['0'] = {'0', '-', 0x0A}, /**/ ['1'] = {'1', '-', 0x0A}},

    [0x0A] = {['0'] = {'0', 'E',    0}, /**/ ['1'] = {'1', 'L', 0x0B}},
    [0x0B] = {['0'] = {'0', 'R', 0x0C}, /**/ ['1'] = {'1', 'R', 0x1A}},
    [0x0C] = {['0'] = {'0', 'E',    0}, /**/ ['1'] = {'1', 'R', 0x0D}},
    [0x0D] = {['0'] = {'0', 'R', 0x0E}, /**/ ['1'] = {'1', 'E',    0}},
    [0x0E] = {['0'] = {'1', 'H',    0}, /**/ ['1'] = {'1', 'E',    0}},

    [0x1A] = {['0'] = {'0', 'E',    0}, /**/ ['1'] = {'0', 'R', 0x1B}},
    [0x1B] = {['0'] = {'0', 'R', 0x1C}, /**/ ['1'] = {'1', 'R', 0x1B}},
    [0x1C] = {['0'] = {'1', 'L', 0x1D}, /**/ ['1'] = {'1', 'E',    0}},
    [0x1D] = {['0'] = {'0', 'L', 0x1E}, /**/ ['1'] = {'1', 'L', 0x1D}},
    [0x1E] = {['0'] = {'0', 'L', 0x1E}, /**/ ['1'] = {'1', 'L', 0x2A}},

    [0x2A] = {['0'] = {'0', 'R', 0x3A}, /**/ ['1'] = {'1', 'R', 0x2B}},
    [0x2B] = {['0'] = {'0', 'E',    0}, /**/ ['1'] = {'0', 'R', 0x2C}},
    [0x2C] = {['0'] = {'0', 'R', 0x2C}, /**/ ['1'] = {'1', 'R', 0x2D}},
    [0x2D] = {['0'] = {'1', 'L', 0x2E}, /**/ ['1'] = {'1', 'R', 0x2D}},
    [0x2E] = {['0'] = {'0', 'L', 0x2F}, /**/ ['1'] = {'1', 'L', 0x2E}},
    [0x2F] = {['0'] = {'0', 'L', 0x2F}, /**/ ['1'] = {'1', 'L', 0x2A}},

    [0x3A] = {['0'] = {'0', 'E',    0}, /**/ ['1'] = {'1', 'R', 0x3B}},
    [0x3B] = {['0'] = {'0', 'R', 0x3C}, /**/ ['1'] = {'1', 'R', 0x3B}},
    [0x3C] = {['0'] = {'0', 'R', 0x3C}, /**/ ['1'] = {'1', 'L', 0x3D}},
    [0x3D] = {['0'] = {'0', 'L', 0x3E}, /**/ ['1'] = {'1', 'E',    0}},
    [0x3E] = {['0'] = {'0', '-', 0x40}, /**/ ['1'] = {'1', 'E',    0}},

    [0x40] = {['0'] = {'1', 'L', 0x40}, /**/ ['1'] = {'1', 'R', 0x4A}},
    [0x4A] = {['0'] = {'0', 'R', 0x4B}, /**/ ['1'] = {'1', 'R', 0x4A}},
    [0x4B] = {['0'] = {'1', 'H',    0}, /**/ ['1'] = {'1', 'R', 0x4B}},
};

struct md Machtypes[] = {
    {"copy",  m_copy},
    {"copy2", m_copy2},
    {"adder", m_adder},
    {NULL},
};
const size_t MACHTYPES_NUM =
 sizeof(Machtypes) / sizeof(typeof(*Machtypes)) - 1;

//==[ End of file ]============================================================
