/*=============================================================================
Turing Machine Debugger
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
  -- License restrictions apply (Dual FAL/GPL2)
  -- For details see doc/FAL.txt and doc/GPL2.txt.
=============================================================================*/
#ifndef SIM_HPP
#define SIM_HPP 1

#include "wx/wx.h"

#include "mach.hpp"
#include "tape.hpp"

class Simulator : public wxFrame {
  public:
    Simulator(const char *, const wxPoint &, const wxSize &);

  protected:
    Tape *tape;
    Machine *mach;

    wxStaticText *t_data, *t_head;
    wxTextCtrl *m_state, *m_until, *m_itv, *i_data;
    wxCheckBox *m_hex;
    wxChoice *i_mt;
    wxTimer *m_timer;
    int m_stop;

  private:
    enum {
        SOP_INIT = 1,
        SOP_STEP,
        SOP_RUN,
        SOP_RUN_TIMER,
    };

    void cb_init(wxCommandEvent &);
    void cb_run(wxCommandEvent &);
    void do_run(wxTimerEvent &);
    void cb_step(wxCommandEvent &);
    int  do_step(void);
    void show_next_step(void);
    void update_tape(void);

    DECLARE_EVENT_TABLE();
};

#endif // SIM_HPP

//==[ End of file ]============================================================
